package com.eco.plugin.xx.hqctb.db.controller;

import com.eco.plugin.xx.hqctb.db.bean.DeptEntity;
import com.eco.plugin.xx.hqctb.db.bean.UserEntity;
import com.eco.plugin.xx.hqctb.db.dao.DeptDao;
import com.eco.plugin.xx.hqctb.db.dao.UserDao;
import com.eco.plugin.xx.hqctb.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;
import com.fr.third.jgroups.util.UUID;

import java.util.List;

/**
 * @author xx
 * @version 10.0
 * Created by xx on 2021-07-29
 **/
@EnableMetrics
@FunctionRecorder
public class DBController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                UserDao.DAO, DeptDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        DBController.accessor = accessor;
    }


    /**
     * 批量操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batch(List<UserEntity> addOrUpdate, List<UserEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final UserEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            UserEntity ae =context.getDAO(UserDao.class).getByUsername(dbe.getUSERNAME());
                            if(ae != null ){
                                dbe.setId(ae.getId());
                                context.getDAO(UserDao.class).update(dbe);
                            }else{
                                context.getDAO(UserDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final UserEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(UserDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }


    /**
     * 单个操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean singleUser(UserEntity addOrUpdate,UserEntity delete) throws Exception {
        //新增或者删除
        if(addOrUpdate != null ){

            accessor.runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    UserEntity ae =context.getDAO(UserDao.class).getByUsername(addOrUpdate.getUSERNAME());
                    if(ae != null ){
                        addOrUpdate.setId(ae.getId());
                        context.getDAO(UserDao.class).update(addOrUpdate);
                    }else{
                        addOrUpdate.setId(UUID.randomUUID().toString());
                        context.getDAO(UserDao.class).add(addOrUpdate);
                    }
                    return true;
                }
            });

        }

        if(delete != null ){
            accessor.runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    UserEntity ae =context.getDAO(UserDao.class).getByUsername(addOrUpdate.getUSERNAME());
                    context.getDAO(UserDao.class).remove(ae.getId());
                    return true;
                }
            });
        }

        return true;
    }



    /**
     * 单个操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean singleDept(DeptEntity addOrUpdate,DeptEntity delete) throws Exception {
        //新增或者删除
        if(addOrUpdate != null ){

            accessor.runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    DeptEntity ae =context.getDAO(DeptDao.class).getByCode(addOrUpdate.getCODE());
                    if(ae != null ){
                        addOrUpdate.setId(ae.getId());
                        context.getDAO(DeptDao.class).update(addOrUpdate);
                    }else{
                        addOrUpdate.setId(UUID.randomUUID().toString());
                        context.getDAO(DeptDao.class).add(addOrUpdate);
                    }
                    return true;
                }
            });

        }

        if(delete != null ){
            accessor.runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    DeptEntity ae =context.getDAO(DeptDao.class).getByCode(delete.getCODE());

                    context.getDAO(DeptDao.class).remove(ae.getId());
                    return true;
                }
            });

        }

        return true;
    }

}
